
package example;

import javax.microedition.midlet.*;
import javax.microedition.lcdui.*;
import javax.microedition.io.*;
import javax.bluetooth.*;
import java.io.*;

/**
 * Discovers a serial port.
 */ 
public class Blue
    extends MIDlet
    implements DiscoveryListener, CommandListener
{    
    
    java.util.Vector devs;
    java.util.Vector servs;
    
    public Blue()
    {
        Form form = new Form( getAppProperty( "Title" ) );
        form.append( getAppProperty( "Message" ) );
        Display.getDisplay(this).setCurrent( form );
	
	devs = new java.util.Vector();
    }
    
    /* When the MIDlet is started, start a device discovery. The
    ** discovery agent will call us back as it discovers devices.
    */
    public void startApp()
    {
	try {
	    javax.bluetooth.LocalDevice.getLocalDevice().getDiscoveryAgent().startInquiry(DiscoveryAgent.GIAC, this);
	} catch (Throwable e) {
	    sherr(e);
	}
    }
    
    /* This is a handy(?) routine which just displays an exception */
    void sherr(Throwable e)
    {
	Form form = new Form( "Exception" );
	form.append( e.toString() );
	Display.getDisplay(this).setCurrent( form );
    }
    
    /* We don't do anything special when the MIDlet is paused or destroyed */
    public void pauseApp()
    {
    }
    
    public void destroyApp( boolean unconditional )
    {
    }
    
    // interface DiscoveryListener
    
    /* This is called whenever the DiscoveryAgent discovers a
    ** nearby Bluetooth device during device discovery. All we
    ** do is put the discovered devices in a list, and display a
    ** running count of the number of devices found so far. */
    public void deviceDiscovered(RemoteDevice btDevice,
				 DeviceClass cod)
    {
	devs.addElement(btDevice);
	Form form = new Form( "Searching" );
	form.append( "Found " + Integer.toString(devs.size()).toString() + " devices" );
	Display.getDisplay(this).setCurrent( form );
    }
    
    /* When discovery is done, display a list of devices and let the
    ** user choose one. Perform a service inquiry on the device the user
    ** chose. */
    public void inquiryCompleted(int discType)
    {
	devs.trimToSize();
	
	/* create a list widget */
	List choose = new List( Integer.toString(devs.size()).toString() + " devices", List.IMPLICIT );
	
	try {
	    int i;
	    
	    /* getting the FriendlyName takes a few seconds per
	    ** device, so display the device addresses first, and then
	    ** go through and look up the name of each one */
	    
	    for(i = 0; i < devs.size(); i++) {
		RemoteDevice d = (RemoteDevice)devs.elementAt(i);
		choose.append(d.getBluetoothAddress(), null);
	    }
	    Display.getDisplay(this).setCurrent( choose );
	    
	    for(i = 0; i < devs.size(); i++) {
		RemoteDevice d = (RemoteDevice)devs.elementAt(i);
		choose.set(i, d.getFriendlyName(false), null);
	    }
	    
	    /* tell the list to call us back when the user selects something */
	    choose.setCommandListener(this);
	} catch (Throwable e) {
	    sherr(e);
	}
    }
    
    // interface CommandListener
    
    public void commandAction( Command aCommand, Displayable aDisplayable )
    {
        /* We only act as the command listener for one UI element, the
        ** list of devices, so that's the only command we handle here */
    
	try {
	    
	    List choose = (List)aDisplayable;
	    
	    /* Get the selected device from the devices list */
	    RemoteDevice d = (RemoteDevice)devs.elementAt(choose.getSelectedIndex());
	    
	    /* Inquire about Serial Port services (UUID 0x1101) */
	    UUID[] query = new UUID[1];
	    query[0] = new UUID(0x1101);
	    servs = new java.util.Vector();
	    
	    javax.bluetooth.LocalDevice.getLocalDevice().getDiscoveryAgent().searchServices(null, query, d, this);
	} catch (Throwable e) {
	    sherr(e);
	}
	
    }

    // interface DiscoveryListener (again)

    /* This is called back as services are discovered */
    public void servicesDiscovered(int transID,
				   ServiceRecord[] servRecord)
    {
	int i;
	for(i = 0; i < servRecord.length; i++)
	    servs.addElement(servRecord[i]);
	
	Form form = new Form( "Searching" );
	form.append( "Found " + Integer.toString(servs.size()).toString() + " services\n" );
	Display.getDisplay(this).setCurrent( form );
    }
    
    /* Once the service search is complete, display them all, then
    ** connect to the first one */
    public void serviceSearchCompleted(int transID,
				       int respCode)
    {
	servs.trimToSize();
	
	Form form = new Form( "Complete" );
	String f = "Found " + Integer.toString(servs.size()).toString() + " services";
	form.append( f );
	Display.getDisplay(this).setCurrent( form );
	
	try {
	    
	    /* List all the services and their names (attribute 0x100) */
	    
	    int[] aids = new int[1];  // attribute IDs
	    aids[0] = 0x0100;
	    
	    int i;
	    for(i = 0; i < servs.size(); i++) {
		ServiceRecord s = (ServiceRecord)servs.elementAt(i);
		form.append(s.getConnectionURL(0, false) + "\n");
		s.populateRecord(aids);
		form.append(s.getAttributeValue(0x0100).toString() + "\n");
	    }

            /* Okay, enough dilly-dallying. Connect to something! */
	    StreamConnection c = (StreamConnection)Connector.open(((ServiceRecord)(servs.elementAt(0))).getConnectionURL(0, false));


	    form.append( c.toString() );

            /* Create input and output streams for the
            ** connection we just made */

	    synchronized(this) {
	    OutputStream os = c.openOutputStream();
	    OutputStreamWriter ow = new OutputStreamWriter(os, "US-ASCII");
	    InputStream is = c.openInputStream();
	    InputStreamReader ir = new InputStreamReader(is, "US-ASCII");

            /* From here on, the MIDlet is talking to the microcontroller
            ** attached to my BlueSMiRF, and the data is probably not very
            ** interesting to anyone else. */
            
	    while(form.size() > 0)
		form.delete(0);

	    form.append(foo(ow, ir, ""));
	    wait(200);
	    form.append(foo(ow, ir, ""));
	    wait(400);
	    form.append(foo(ow, ir, ""));
	    wait(400);

	    form.append(foo(ow, ir, "\n\nQ"));
	    wait(400);

	    form.append(foo(ow, ir, "z0000"));
	    wait(400);

	    form.append(foo(ow, ir, "Q"));
	    wait(400);

	    form.append(foo(ow, ir, ""));
	    wait(400);

	    form.append(foo(ow, ir, "Q"));
	    wait(400);

	    form.append(foo(ow, ir, ""));

            /* Close the connection */
            
	    ow.close();
	    ir.close();
	    c.close();
	    }
	    
	    form.append("done");
	    
	} catch (Throwable e) {
	    form.append(" ERR: " + e.toString() );
	}
	
    }

    /* This writes a string to an OutputStreamWriter, reads anything
    ** that's immediately available from the InputStreamReader, and
    ** returns a string which can be printed to debug the traffic */
    String foo(OutputStreamWriter w, InputStreamReader r, String s)
	throws java.io.IOException
    {
	String a;
	StringBuffer b = new StringBuffer();

	a = bar(r);
	if (a.length() > 0) {
	    b.append("<< ");
	    b.append(a);
	    b.append('\n');
	}
	w.write(s);
	w.flush();
	if (s.length() > 0) {
	    b.append(">> ");
	    b.append(s);
	    b.append('\n');
	}
	a = bar(r);
	if (a.length() > 0) {
	    b.append("<< ");
	    b.append(a);
	    b.append('\n');
	}

	return b.toString();
    }

    /* This returns whatever is immediately available from the input
    ** stream, without blocking */
    String bar(Reader r)
	throws java.io.IOException
    {
	StringBuffer b = new StringBuffer();
	
	while(r.ready()) {
	    b.append((char)(r.read()));
	}

	return b.toString();
    }
    
}

